# IBM Confidential
# PID 5900-BAF
# Copyright StreamSets Inc., an IBM Company 2025

"""Model module."""

import collections
from copy import deepcopy
from ibm_watsonx_data_integration.common.constants import EXPOSE_SUB_CLASS, HIDDEN_DICTIONARY, DataActions
from ibm_watsonx_data_integration.common.utils import SeekableList, TraversableDict, matches_filters
from operator import attrgetter
from pydantic import BaseModel as PydanticBaseModel
from pydantic import ConfigDict, SerializationInfo, model_serializer, model_validator
from typing import Any, ClassVar


class BaseModel(PydanticBaseModel):
    """BaseModel class to standardize objects ."""

    # Use this var to control exposing nested dictionaries from you data
    # The order in which the keys are provided matter
    # Base level IGNORE will happen first
    EXPOSED_DATA_PATH: ClassVar[dict] = {}
    """"Dictionary of key paths used to expose data as attributes

    :meta private:
    """

    # Pydantic variable
    model_config = ConfigDict(
        extra="allow",  # Allow extra fields in the base layer of the data to be attributes
        arbitrary_types_allowed=True,  # Allow non pydantic BaseModel classes to be included in the model
        use_enum_values=True,  # Populate models with the value property of enums,
        #  rather than the raw enum (on model dump)
    )

    def __setattr__(self, name: str, value: Any) -> None:  # noqa: ANN401
        """Setter for attributes.

        Arguments:
           name: Name of the attribute to set
           value: Value of the attribute
        """
        if name in self.__pydantic_extra__:
            self.model_config["extra"] = "allow"

        try:
            super().__setattr__(name, value)
        finally:
            self.model_config["extra"] = "forbid"

    def __repr__(self) -> str:
        """Representation of object.

        Returns:
              A :py:obj:`str` representing the object.
        """
        if hasattr(self, EXPOSE_SUB_CLASS):
            if not getattr(self, EXPOSE_SUB_CLASS):
                return self.__repr_str__(" ")  # Returns __repr__ w/o class name
        return super().__repr__()

    def __str__(self) -> str:
        """String representation of object.

        Returns:
              A :py:obj:`str` representing the object.
        """
        return f"{self.__class__.__name__}({super().__str__()})"

    def __repr_args__(self) -> list:
        """Gets a list of attributes that represent the object.

        Returns:
              A :py:obj:`list` containing attributes that represent the object.
        """
        # Taken from pydantic
        attrs_names = []
        for i in self.__pydantic_fields__:
            if self.__pydantic_fields__[i].repr:
                attrs_names.append(i)
        attrs = ((s, getattr(self, s)) for s in attrs_names)

        # Check if attrs has _expose and is set to False
        # If _expose is set to False don't show class name
        attr_args = []
        for a, v in attrs:
            if v is not None:
                if hasattr(v, EXPOSE_SUB_CLASS) and not v._expose:
                    attr_args.extend(v.__repr_args__())
                elif v is not self:
                    attr_args.append((a, v))
                else:
                    attr_args.append((a, self.__repr_recursion__(v)))
        return attr_args

    @model_validator(mode="before")
    @classmethod
    def entity_validation(cls, data: dict) -> dict:
        """Flattens/Manipulates the data.

        Arguments:
           data: Data to be manipulated on

        Returns:
              A :py:obj:`dict` with manipulated data.
        """

        def _flatten_from_key_path(data: dict, key_path: str, key_path_value: dict) -> None:
            data[HIDDEN_DICTIONARY][key_path] = {}
            keys = key_path.split(".")
            expose_keys = key_path_value.get(DataActions.EXPOSE, None)
            ignore_keys = key_path_value.get(DataActions.IGNORE, None)
            tmp_data = data

            # Traverse to the key depth
            for key in keys[:-1]:
                tmp_data = tmp_data[key]
            tmp_data = tmp_data.pop(keys[-1])

            # Keep track of what keys belong
            if expose_keys:
                ignore_keys = tmp_data.keys() - expose_keys
            elif ignore_keys:
                expose_keys = tmp_data.keys() - ignore_keys
            else:
                expose_keys = tmp_data.keys()
                ignore_keys = []

            data[HIDDEN_DICTIONARY][key_path]["exposed_keys"] = list(expose_keys)
            data[HIDDEN_DICTIONARY][key_path]["ignored"] = {}
            # These keys overwrite data exposed in the base layer, need to be maintained
            # so we don't delete it from the base layer
            data[HIDDEN_DICTIONARY][key_path]["overwrite"] = []

            # Flatten out the data
            for key in expose_keys:
                if key in data.keys():
                    data[HIDDEN_DICTIONARY][key_path]["overwrite"].append(key)
                data[key] = tmp_data[key]
            for key in ignore_keys:
                data[HIDDEN_DICTIONARY][key_path]["ignored"][key] = tmp_data[key]

        def _hide_data(data: dict, keys_to_hide: list) -> None:
            for key in keys_to_hide:
                data[HIDDEN_DICTIONARY]["base_keys"][key] = data[key]
                del data[key]

        if HIDDEN_DICTIONARY not in data:
            data[HIDDEN_DICTIONARY] = {}

        key_paths = list(cls.EXPOSED_DATA_PATH.keys())

        # Check for base level actions
        data = deepcopy(data)  # So we don't manipulate the original data
        if DataActions.IGNORE in key_paths:
            key_paths.remove(DataActions.IGNORE)
            data[HIDDEN_DICTIONARY]["base_keys"] = {}
            _hide_data(data, cls.EXPOSED_DATA_PATH[DataActions.IGNORE])

        for key_path in key_paths:
            key_action = cls.EXPOSED_DATA_PATH[key_path]
            _flatten_from_key_path(data, key_path, key_action)
        return data

    @model_serializer(mode="wrap")
    def serialize_model(self, handler: Any, info: SerializationInfo) -> dict:  # noqa: ANN401
        """Serializes the data, back into it's original formatting.

        Returns:
              A :py:obj:`dict` with updated information
        """
        data = handler(self)
        base_keys = data[HIDDEN_DICTIONARY].get("base_keys", None)
        if base_keys:
            for key in base_keys.keys():
                data[key] = data[HIDDEN_DICTIONARY]["base_keys"][key]
            del data[HIDDEN_DICTIONARY]["base_keys"]

        for key_path in data[HIDDEN_DICTIONARY]:
            keys = key_path.split(".")
            tmp_data = data

            # Traverse through the keys
            for key in keys:
                tmp_data.setdefault(key, {})
                tmp_data = tmp_data[key]

            # Wrap back the data in its appropriate field
            for key in data[HIDDEN_DICTIONARY][key_path]["exposed_keys"]:
                tmp_data[key] = data[key]
                if key not in data[HIDDEN_DICTIONARY][key_path]["overwrite"]:
                    del data[key]
            tmp_data.update(data[HIDDEN_DICTIONARY][key_path]["ignored"])
        del data[HIDDEN_DICTIONARY]
        return data

    def model_dump(self, by_alias: bool = True, **kwargs: any) -> dict:
        """Generate a dictionary representation of the model, optionally specifying which fields to include or exclude.

        Returns:
           A dictionary representation of the model.
        """
        return super().model_dump(by_alias=by_alias, **kwargs)

    def model_post_init(self, __context: Any) -> None:  # Automatically runs post init # noqa: ANN401
        """Don't allow extra variables after initialization."""
        self.model_config["extra"] = "ignore"  # Doesn't allow extra variables to be set


CollectionModelResults = collections.namedtuple(
    "CollectionModelResults",
    ["results", "class_type", "response_bookmark", "request_bookmark", "response_location", "constructor_params"],
)


class CollectionModel:
    """Base class wrapper with abstractions for pagination."""

    def __init__(self, platform: object = None) -> None:
        """The __init__ of the Project class.

        Args:
            platform: The Platform object.
        """
        self._platform = platform

    def _paginate(self, **kwargs: dict) -> BaseModel:
        """Allows fetching of items in batches (pages).

        Returns:
            An inherited instance of :py:class:`ibm_watsonx_data_integration.common.models.BaseModel`.
        """
        filters = kwargs
        local_filters = {key: value for key, value in filters.items() if key not in self._request_parameters()}
        all_ids = set()
        retriever = attrgetter(self.unique_id)
        current_results_len = len(self)

        # Iterate over pages
        while current_results_len > 0:
            response, class_type, response_bookmark, request_bookmark, response_location, constructor_params = (
                self._get_results_from_api(filters)
            )
            response = TraversableDict(response)
            current_results = list(class_type(**constructor_params, **item) for item in response[response_location])
            for result in current_results:
                # This check is to avoid duplicates
                item_id = retriever(result)
                if (item_id not in all_ids) and matches_filters(result, **local_filters):
                    all_ids.add(item_id)
                    yield result

            if response.get(response_bookmark, None):
                filters[request_bookmark] = response[response_bookmark]
            else:
                break

            current_results_len = len(current_results)

    def __repr__(self) -> str:
        """List representation of the objects.

        Returns:
              A :py:obj:`str` representing the objects.
        """
        return str([item for item in self._paginate()])

    def __iter__(self) -> BaseModel:
        """Iterate over the objects obtained.

        Returns:
            An inherited instance of :py:class:`ibm_watsonx_data_integration.common.models.BaseModel`.
        """
        for item in self._paginate():
            yield item

    def __len__(self) -> int:
        """Provides length (count) of items."""
        raise NotImplementedError(f"Function has not been implemented for class {type(self).__name__}")

    def __getitem__(self, i: int) -> BaseModel:
        """Enables the user to fetch items by index.

        Args:
            i (:obj:`int`): Index of the item.

        Returns:
            An inherited instance of :py:class:`ibm_watsonx_data_integration.common.models.BaseModel`.
        """
        return list(self._paginate())[i]

    def _get_all_results_from_api(self, **kwargs: dict) -> CollectionModelResults:
        """Used to get multiple (all) results from api.

        Args:
            **kwargs: Optional arguments to be passed to filter the results.

        Returns:
            A :obj:`collections.namedtuple`: of
                results (:obj:`list`): a list of inherited instances of :py:class:`streamsets.sdk.sch_models.BaseModel`
                class_type (:py:class:`streamsets.sdk.sch_models.BaseModel`): the type of class to instantiate
                response_bookmark (:obj:`str`): the location of the bookmark in the results
                request_bookmark (:obj:`str`): the location of where to put the bookmark when making a new request
                response_location (:obj:`str`): the location of the results needed to create object in the results
        """
        raise NotImplementedError(f"Function has not been implemented for class {type(self).__name__}")

    def _request_parameters(self) -> list:
        """Get a list of params that is used when making a api request."""
        raise NotImplementedError(f"Function has not been implemented for class {type(self).__name__}")

    def get_all(self, **kwargs: dict) -> list:
        """Used to get multiple (all) results from api.

        Args:
            **kwargs: Optional other arguments to be passed to filter the results.

        Returns:
            A :py:obj:`list` of inherited instances of
                :py:class:`streamsets.sdk.sch_models.BaseModel`.
        """
        return SeekableList(self._paginate(**kwargs))

    def get(self, **kwargs: dict) -> BaseModel:
        """Used to get an instant result from the api.

        Args:
            **kwargs: Optional arguments to be passed to filter the results.

        Returns:
            An inherited instance of :py:class:`streamsets.sdk.sch_models.BaseModel`.

        Raises:
            ValueError: If instance is not in the list.
        """
        for item in self._paginate(**kwargs):
            return item
        # Raise instance doesn't exist if not found at the end
        raise ValueError("Instance ({}) is not in list".format(", ".join(f"{k}={v}" for k, v in kwargs.items())))



# IBM Confidential
# PID 5900-BAF
# Copyright StreamSets Inc., an IBM Company 2025

"""Flows module."""

# Can't be part of flow_model.py due to circular imports
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.cpd_models.project_model import Project
import itertools
from ibm_watsonx_data_integration.common.models import BaseModel, CollectionModel
from ibm_watsonx_data_integration.common.utils import SeekableList
from ibm_watsonx_data_integration.services.datastage.models import (
    DataStageFlows,
)
from ibm_watsonx_data_integration.services.streamsets.models import (
    StreamsetsFlows,
)


class Flows(CollectionModel):
    """Collection of DataStageFlow and StreamsetsFlow objects."""

    def __init__(self, project: "Project") -> None:
        """The __init__ of the Flows class.

        Args:
            project: The Project object.
        """
        self._sx_flows = StreamsetsFlows(project)
        self._data_flows = DataStageFlows(project)

    def _paginate(self, **kwargs: dict) -> BaseModel:
        for item in itertools.chain(self._sx_flows, self._data_flows):
            yield item

    def get_all(self, flow_type: str = None, **kwargs: dict) -> SeekableList:
        """Used to get multiple (all) results from flows api.

        Args:
            flow_type: The type of flow to be returned
            **kwargs: Optional other arguments to be passed to filter the results.

        Returns:
            A :py:obj:`list` of inherited instances of
                :py:class:`streamsets.sdk.sch_models.BaseModel`.
        """
        if flow_type == "streamsets":
            return self._sx_flows.get_all(**kwargs)
        elif flow_type == "datastage":
            return self._data_flows.get_all(**kwargs)
        else:
            return self._sx_flows.get_all(**kwargs) + self._data_flows.get_all(**kwargs)

    def get(self, flow_type: str = None, **kwargs: dict) -> SeekableList:
        """Used to get an instant result from the api.

        Args:
            flow_type: The type of flow to be returned
            **kwargs: Optional arguments to be passed to filter the results.

        Returns:
            An inherited instance of :py:class:`streamsets.sdk.sch_models.BaseModel`.

        Raises:
            ValueError: If instance is not in the list.
        """
        if flow_type == "streamsets":
            return self._sx_flows.get(**kwargs)
        elif flow_type == "datastage":
            return self._data_flows.get(**kwargs)
        else:
            try:
                return self._sx_flows.get(**kwargs)
            except Exception:
                return self._data_flows.get(**kwargs)



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025

"""Module containing Service ID Model."""

from ibm_watsonx_data_integration.common.models import BaseModel, CollectionModel, CollectionModelResults
from pydantic import Field
from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.platform import Platform


class ServiceID(BaseModel):
    """Model representing a Service ID."""

    id: str = Field(repr=True, frozen=True)
    name: str = Field(repr=True)
    iam_id: str = Field(repr=False, frozen=True)
    account_id: str = Field(repr=False, frozen=True)
    entity_tag: str = Field(repr=False, frozen=True)
    crn: str = Field(repr=False, frozen=True)
    locked: bool = Field(repr=False)
    created_at: str = Field(repr=False, frozen=True)
    modified_at: str = Field(repr=False)

    def __init__(self, platform: Optional["Platform"] = None, **service_id_json: dict) -> None:
        """The __init__ of the ServiceID Wrapper class.

        Args:
            service_id_json: The JSON for the Service ID.
            platform: The Platform object. Default: ``None``
        """
        super().__init__(**service_id_json)
        self._platform = platform

    @property
    def type(self) -> str:
        """This property returns the member type "service".

        Returns:
            The member type.
        """
        return "service"


class ServiceIDs(CollectionModel):
    """Collection of ServiceID instances."""

    def __init__(self, platform: Optional["Platform"] = None) -> None:
        """The __init__ of the ServiceIDs class.

        Args:
            platform: The Platform object.
        """
        super().__init__(platform)
        self.unique_id = "id"

    def __len__(self) -> int:
        """Total amount of service IDs."""
        self._platform.current_account
        list_of_service_ids = self._platform._service_id_api.list_all_service_ids(
            params={"account_id": self._platform._current_account.account_id}
        ).json()["serviceids"]
        return len(list_of_service_ids)

    def _request_parameters(self) -> list:
        return [
            "account_id",
            "group_id",
            "name",
            "pagesize",
            "pagetoken",
            "sort",
            "order",
            "include_history",
            "filter",
            "show_group_id",
        ]

    def _get_results_from_api(self, request_params: dict = None, **kwargs: dict) -> CollectionModelResults:
        """Returns results of a ServiceID API request to list_all_service_ids."""
        page_token = None
        if "account_id" in request_params:
            next_url = request_params.pop("account_id")
            page_token = next_url.split("pagetoken=")[1]

        request_params_defaults = {
            "account_id": self._platform._current_account.account_id,
            "group_id": None,
            "name": None,
            "pagesize": 100,
            "pagetoken": page_token,
            "sort": None,
            "order": None,
            "include_history": None,
            "filter": None,
            "show_group_id": None,
        }
        request_params_unioned = request_params_defaults
        request_params_unioned.update(request_params)
        response = self._platform._service_id_api.list_all_service_ids(
            params={k: v for k, v in request_params_unioned.items() if v is not None}
        ).json()
        return CollectionModelResults(
            response,
            ServiceID,
            "next",
            "account_id",
            "serviceids",
            {"platform": self._platform},
        )



# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

from typing import TYPE_CHECKING, Union, Literal, overload
import ibm_watsonx_data_integration.services.datastage.models.stages as stage
from ibm_watsonx_data_integration.services.datastage.models.flow.dag import DAG
from ibm_watsonx_data_integration.services.datastage.models.flow.string_to_stage import string_to_stage_class
from ibm_watsonx_data_integration.services.datastage.models.stage_type_str import StageTypeStr
from typing import Literal, overload



class FlowComposer:
    def __init__(self, dag: "DAG"):
        self._dag = dag


    @overload
    def add_stage(self, type: Literal["Address Verification"], label: str = None) -> stage.AddressVerificationStage: ...

    @overload
    def add_stage(self, type: Literal["Aggregator"], label: str = None) -> stage.AggregatorStage: ...

    @overload
    def add_stage(self, type: Literal["Amazon RDS for MySQL"], label: str = None) -> stage.AmazonRDSForMysqlStage: ...

    @overload
    def add_stage(self, type: Literal["Amazon RDS for Oracle"], label: str = None) -> stage.AmazonRDSForOracleStage: ...

    @overload
    def add_stage(self, type: Literal["Amazon RDS for PostgreSQL"], label: str = None) -> stage.AmazonRDSForPostgresqlStage: ...

    @overload
    def add_stage(self, type: Literal["Amazon Redshift"], label: str = None) -> stage.AmazonRedshiftStage: ...

    @overload
    def add_stage(self, type: Literal["Amazon S3"], label: str = None) -> stage.AmazonS3Stage: ...

    @overload
    def add_stage(self, type: Literal["Apache Cassandra"], label: str = None) -> stage.ApacheCassandraStage: ...

    @overload
    def add_stage(self, type: Literal["Apache Cassandra for DataStage"], label: str = None) -> stage.ApacheCassandraForDatastageStage: ...

    @overload
    def add_stage(self, type: Literal["Apache Derby"], label: str = None) -> stage.ApacheDerbyStage: ...

    @overload
    def add_stage(self, type: Literal["Apache HBase"], label: str = None) -> stage.ApacheHbaseStage: ...

    @overload
    def add_stage(self, type: Literal["Apache HDFS"], label: str = None) -> stage.ApacheHDFSStage: ...

    @overload
    def add_stage(self, type: Literal["Apache Hive"], label: str = None) -> stage.ApacheHiveStage: ...

    @overload
    def add_stage(self, type: Literal["Apache Impala"], label: str = None) -> stage.ApacheImpalaStage: ...

    @overload
    def add_stage(self, type: Literal["Apache Kafka"], label: str = None) -> stage.ApacheKafkaStage: ...

    @overload
    def add_stage(self, type: Literal["Azure PostgreSQL"], label: str = None) -> stage.AzurePostgresqlStage: ...

    @overload
    def add_stage(self, type: Literal["Bloom Filter"], label: str = None) -> stage.BloomFilterStage: ...

    @overload
    def add_stage(self, type: Literal["Box"], label: str = None) -> stage.BoxStage: ...

    @overload
    def add_stage(self, type: Literal["Change Apply"], label: str = None) -> stage.ChangeApplyStage: ...

    @overload
    def add_stage(self, type: Literal["Change Capture"], label: str = None) -> stage.ChangeCaptureStage: ...

    @overload
    def add_stage(self, type: Literal["Checksum"], label: str = None) -> stage.ChecksumStage: ...

    @overload
    def add_stage(self, type: Literal["Column Export"], label: str = None) -> stage.ColumnExportStage: ...

    @overload
    def add_stage(self, type: Literal["Column Generator"], label: str = None) -> stage.ColumnGeneratorStage: ...

    @overload
    def add_stage(self, type: Literal["Column Import"], label: str = None) -> stage.ColumnImportStage: ...

    @overload
    def add_stage(self, type: Literal["Combine Records"], label: str = None) -> stage.CombineRecordsStage: ...

    @overload
    def add_stage(self, type: Literal["Compare"], label: str = None) -> stage.CompareStage: ...

    @overload
    def add_stage(self, type: Literal["Complex Flat File"], label: str = None) -> stage.ComplexFlatFileStage: ...

    @overload
    def add_stage(self, type: Literal["Compress"], label: str = None) -> stage.CompressStage: ...

    @overload
    def add_stage(self, type: Literal["Copy"], label: str = None) -> stage.CopyStage: ...

    @overload
    def add_stage(self, type: Literal["Data Rules"], label: str = None) -> stage.DataRulesStage: ...

    @overload
    def add_stage(self, type: Literal["Data set"], label: str = None) -> stage.DataSetStage: ...

    @overload
    def add_stage(self, type: Literal["DataStax Enterprise"], label: str = None) -> stage.DatastaxEnterpriseStage: ...

    @overload
    def add_stage(self, type: Literal["Decode"], label: str = None) -> stage.DecodeStage: ...

    @overload
    def add_stage(self, type: Literal["Denodo"], label: str = None) -> stage.DenodoStage: ...

    @overload
    def add_stage(self, type: Literal["Difference"], label: str = None) -> stage.DifferenceStage: ...

    @overload
    def add_stage(self, type: Literal["Dremio"], label: str = None) -> stage.DremioStage: ...

    @overload
    def add_stage(self, type: Literal["Dropbox"], label: str = None) -> stage.DropboxStage: ...

    @overload
    def add_stage(self, type: Literal["Elasticsearch"], label: str = None) -> stage.ElasticsearchStage: ...

    @overload
    def add_stage(self, type: Literal["Encode"], label: str = None) -> stage.EncodeStage: ...

    @overload
    def add_stage(self, type: Literal["Exasol"], label: str = None) -> stage.ExasolStage: ...

    @overload
    def add_stage(self, type: Literal["Expand"], label: str = None) -> stage.ExpandStage: ...

    @overload
    def add_stage(self, type: Literal["External Filter"], label: str = None) -> stage.ExternalFilterStage: ...

    @overload
    def add_stage(self, type: Literal["External Source"], label: str = None) -> stage.ExternalSourceStage: ...

    @overload
    def add_stage(self, type: Literal["External Target"], label: str = None) -> stage.ExternalTargetStage: ...

    @overload
    def add_stage(self, type: Literal["FTP"], label: str = None) -> stage.FTPStage: ...

    @overload
    def add_stage(self, type: Literal["File set"], label: str = None) -> stage.FileSetStage: ...

    @overload
    def add_stage(self, type: Literal["Filter"], label: str = None) -> stage.FilterStage: ...

    @overload
    def add_stage(self, type: Literal["Funnel"], label: str = None) -> stage.FunnelStage: ...

    @overload
    def add_stage(self, type: Literal["Generic"], label: str = None) -> stage.GenericStage: ...

    @overload
    def add_stage(self, type: Literal["Generic JDBC"], label: str = None) -> stage.GenericJDBCStage: ...

    @overload
    def add_stage(self, type: Literal["Generic S3"], label: str = None) -> stage.GenericS3Stage: ...

    @overload
    def add_stage(self, type: Literal["Google BigQuery"], label: str = None) -> stage.GoogleBigqueryStage: ...

    @overload
    def add_stage(self, type: Literal["Google Cloud Pub/Sub"], label: str = None) -> stage.GoogleCloudPubSubStage: ...

    @overload
    def add_stage(self, type: Literal["Google Cloud Storage"], label: str = None) -> stage.GoogleCloudStorageStage: ...

    @overload
    def add_stage(self, type: Literal["Google Looker"], label: str = None) -> stage.GoogleLookerStage: ...

    @overload
    def add_stage(self, type: Literal["Greenplum"], label: str = None) -> stage.GreenplumStage: ...

    @overload
    def add_stage(self, type: Literal["HTTP"], label: str = None) -> stage.HTTPStage: ...

    @overload
    def add_stage(self, type: Literal["Head"], label: str = None) -> stage.HeadStage: ...

    @overload
    def add_stage(self, type: Literal["Hierarchical Data"], label: str = None) -> stage.HierarchicalDataStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Cloud Databases for MongoDB"], label: str = None) -> stage.IBMCloudDatabasesForMongodbStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Cloud Databases for MySQL"], label: str = None) -> stage.IBMCloudDatabasesForMysqlStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Cloud Databases for PostgreSQL"], label: str = None) -> stage.IBMCloudDatabasesForPostgresqlStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Cloud Object Storage"], label: str = None) -> stage.IBMCloudObjectStorageStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Cognos Analytics"], label: str = None) -> stage.IBMCognosAnalyticsStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Data Virtualization"], label: str = None) -> stage.IBMDataVirtualizationStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Data Virtualization Manager for z/OS"], label: str = None) -> stage.IBMDataVirtualizationManagerForZOSStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Db2"], label: str = None) -> stage.IBMDb2Stage: ...

    @overload
    def add_stage(self, type: Literal["IBM Db2 Big SQL"], label: str = None) -> stage.IBMDb2BigSQLStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Db2 Warehouse"], label: str = None) -> stage.IBMDb2WarehouseStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Db2 for DataStage"], label: str = None) -> stage.IBMDb2ForDatastageStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Db2 for i"], label: str = None) -> stage.IBMDb2ForIStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Db2 for z/OS"], label: str = None) -> stage.IBMDb2ForZOSStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Db2 on Cloud"], label: str = None) -> stage.IBMDb2OnCloudStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Informix"], label: str = None) -> stage.IBMInformixStage: ...

    @overload
    def add_stage(self, type: Literal["IBM MQ"], label: str = None) -> stage.IBMMQStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Match 360"], label: str = None) -> stage.IBMMatch360Stage: ...

    @overload
    def add_stage(self, type: Literal["IBM Netezza Performance Server"], label: str = None) -> stage.IBMNetezzaPerformanceServerStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Netezza Performance Server for DataStage"], label: str = None) -> stage.IBMNetezzaPerformanceServerForDatastageStage: ...

    @overload
    def add_stage(self, type: Literal["IBM Planning Analytics"], label: str = None) -> stage.IBMPlanningAnalyticsStage: ...

    @overload
    def add_stage(self, type: Literal["IBM watsonx.data Presto"], label: str = None) -> stage.IBMWatsonxDataPrestoStage: ...

    @overload
    def add_stage(self, type: Literal["Investigate"], label: str = None) -> stage.InvestigateStage: ...

    @overload
    def add_stage(self, type: Literal["Java Integration"], label: str = None) -> stage.JavaIntegrationStage: ...

    @overload
    def add_stage(self, type: Literal["Join"], label: str = None) -> stage.JoinStage: ...

    @overload
    def add_stage(self, type: Literal["Lookup"], label: str = None) -> stage.LookupStage: ...

    @overload
    def add_stage(self, type: Literal["Lookup file set"], label: str = None) -> stage.LookupFileSetStage: ...

    @overload
    def add_stage(self, type: Literal["Make Subrecord"], label: str = None) -> stage.MakeSubrecordStage: ...

    @overload
    def add_stage(self, type: Literal["Make Vector"], label: str = None) -> stage.MakeVectorStage: ...

    @overload
    def add_stage(self, type: Literal["MariaDB"], label: str = None) -> stage.MariadbStage: ...

    @overload
    def add_stage(self, type: Literal["Match Frequency"], label: str = None) -> stage.MatchFrequencyStage: ...

    @overload
    def add_stage(self, type: Literal["Merge"], label: str = None) -> stage.MergeStage: ...

    @overload
    def add_stage(self, type: Literal["Microsoft Azure Blob Storage"], label: str = None) -> stage.MicrosoftAzureBlobStorageStage: ...

    @overload
    def add_stage(self, type: Literal["Microsoft Azure Cosmos DB"], label: str = None) -> stage.MicrosoftAzureCosmosDBStage: ...

    @overload
    def add_stage(self, type: Literal["Microsoft Azure Data Lake Storage"], label: str = None) -> stage.MicrosoftAzureDataLakeStorageStage: ...

    @overload
    def add_stage(self, type: Literal["Microsoft Azure Databricks"], label: str = None) -> stage.MicrosoftAzureDatabricksStage: ...

    @overload
    def add_stage(self, type: Literal["Microsoft Azure File Storage"], label: str = None) -> stage.MicrosoftAzureFileStorageStage: ...

    @overload
    def add_stage(self, type: Literal["Microsoft Azure SQL Database"], label: str = None) -> stage.MicrosoftAzureSQLDatabaseStage: ...

    @overload
    def add_stage(self, type: Literal["Microsoft Azure Synapse Analytics"], label: str = None) -> stage.MicrosoftAzureSynapseAnalyticsStage: ...

    @overload
    def add_stage(self, type: Literal["Microsoft SQL Server"], label: str = None) -> stage.MicrosoftSQLServerStage: ...

    @overload
    def add_stage(self, type: Literal["MinIO"], label: str = None) -> stage.MinioStage: ...

    @overload
    def add_stage(self, type: Literal["Modify"], label: str = None) -> stage.ModifyStage: ...

    @overload
    def add_stage(self, type: Literal["MongoDB"], label: str = None) -> stage.MongodbStage: ...

    @overload
    def add_stage(self, type: Literal["MySQL"], label: str = None) -> stage.MysqlStage: ...

    @overload
    def add_stage(self, type: Literal["ODBC"], label: str = None) -> stage.ODBCStage: ...

    @overload
    def add_stage(self, type: Literal["One-source Match"], label: str = None) -> stage.OneSourceMatchStage: ...

    @overload
    def add_stage(self, type: Literal["Oracle"], label: str = None) -> stage.OracleStage: ...

    @overload
    def add_stage(self, type: Literal["Oracle Database for DataStage"], label: str = None) -> stage.OracleDatabaseForDatastageStage: ...

    @overload
    def add_stage(self, type: Literal["Peek"], label: str = None) -> stage.PeekStage: ...

    @overload
    def add_stage(self, type: Literal["Pivot Enterprise"], label: str = None) -> stage.PivotEnterpriseStage: ...

    @overload
    def add_stage(self, type: Literal["PostgreSQL"], label: str = None) -> stage.PostgresqlStage: ...

    @overload
    def add_stage(self, type: Literal["Presto"], label: str = None) -> stage.PrestoStage: ...

    @overload
    def add_stage(self, type: Literal["Promote Subrecord"], label: str = None) -> stage.PromoteSubrecordStage: ...

    @overload
    def add_stage(self, type: Literal["Remove Duplicates"], label: str = None) -> stage.RemoveDuplicatesStage: ...

    @overload
    def add_stage(self, type: Literal["Rest"], label: str = None) -> stage.RestStage: ...

    @overload
    def add_stage(self, type: Literal["Row Generator"], label: str = None) -> stage.RowGeneratorStage: ...

    @overload
    def add_stage(self, type: Literal["SAP ASE"], label: str = None) -> stage.SAPASEStage: ...

    @overload
    def add_stage(self, type: Literal["SAP BAPI"], label: str = None) -> stage.SAPBAPIStage: ...

    @overload
    def add_stage(self, type: Literal["SAP Bulk Extract"], label: str = None) -> stage.SAPBulkExtractStage: ...

    @overload
    def add_stage(self, type: Literal["SAP Delta Extract"], label: str = None) -> stage.SAPDeltaExtractStage: ...

    @overload
    def add_stage(self, type: Literal["SAP HANA"], label: str = None) -> stage.SAPHANAStage: ...

    @overload
    def add_stage(self, type: Literal["SAP IDoc"], label: str = None) -> stage.SAPIdocStage: ...

    @overload
    def add_stage(self, type: Literal["SAP IQ"], label: str = None) -> stage.SAPIQStage: ...

    @overload
    def add_stage(self, type: Literal["SAP OData"], label: str = None) -> stage.SAPOdataStage: ...

    @overload
    def add_stage(self, type: Literal["Salesforce API for DataStage"], label: str = None) -> stage.SalesforceAPIForDatastageStage: ...

    @overload
    def add_stage(self, type: Literal["Salesforce.com"], label: str = None) -> stage.SalesforceComStage: ...

    @overload
    def add_stage(self, type: Literal["Sample"], label: str = None) -> stage.SampleStage: ...

    @overload
    def add_stage(self, type: Literal["Sequential file"], label: str = None) -> stage.SequentialFileStage: ...

    @overload
    def add_stage(self, type: Literal["SingleStoreDB"], label: str = None) -> stage.SinglestoredbStage: ...

    @overload
    def add_stage(self, type: Literal["Slowly Changing Dimension"], label: str = None) -> stage.SlowlyChangingDimensionStage: ...

    @overload
    def add_stage(self, type: Literal["Snowflake"], label: str = None) -> stage.SnowflakeStage: ...

    @overload
    def add_stage(self, type: Literal["Sort"], label: str = None) -> stage.SortStage: ...

    @overload
    def add_stage(self, type: Literal["Split Subrecord"], label: str = None) -> stage.SplitSubrecordStage: ...

    @overload
    def add_stage(self, type: Literal["Split Vector"], label: str = None) -> stage.SplitVectorStage: ...

    @overload
    def add_stage(self, type: Literal["Standardize"], label: str = None) -> stage.StandardizeStage: ...

    @overload
    def add_stage(self, type: Literal["Storage volume"], label: str = None) -> stage.StorageVolumeStage: ...

    @overload
    def add_stage(self, type: Literal["Surrogate Key Generator"], label: str = None) -> stage.SurrogateKeyGeneratorStage: ...

    @overload
    def add_stage(self, type: Literal["Survive"], label: str = None) -> stage.SurviveStage: ...

    @overload
    def add_stage(self, type: Literal["Switch"], label: str = None) -> stage.SwitchStage: ...

    @overload
    def add_stage(self, type: Literal["Tableau"], label: str = None) -> stage.TableauStage: ...

    @overload
    def add_stage(self, type: Literal["Tail"], label: str = None) -> stage.TailStage: ...

    @overload
    def add_stage(self, type: Literal["Teradata"], label: str = None) -> stage.TeradataStage: ...

    @overload
    def add_stage(self, type: Literal["Teradata database for DataStage"], label: str = None) -> stage.TeradataDatabaseForDatastageStage: ...

    @overload
    def add_stage(self, type: Literal["Transformer"], label: str = None) -> stage.TransformerStage: ...

    @overload
    def add_stage(self, type: Literal["Two-source Match"], label: str = None) -> stage.TwoSourceMatchStage: ...

    @overload
    def add_stage(self, type: Literal["Vertica"], label: str = None) -> stage.VerticaStage: ...

    @overload
    def add_stage(self, type: Literal["Wave Generator"], label: str = None) -> stage.WaveGeneratorStage: ...

    @overload
    def add_stage(self, type: Literal["Web Service"], label: str = None) -> stage.WebServiceStage: ...

    @overload
    def add_stage(self, type: Literal["Write Range Map"], label: str = None) -> stage.WriteRangeMapStage: ...

    @overload
    def add_stage(self, type: Literal["XML Input"], label: str = None) -> stage.XMLInputStage: ...

    @overload
    def add_stage(self, type: Literal["XML Output"], label: str = None) -> stage.XMLOutputStage: ...

    def add_stage(self, type: StageTypeStr, label: str = None):
        stage_class = string_to_stage_class(type)
        node = stage_class(self._dag, label=label)
        self._dag.add_node(node)
        return node



# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

from typing import TYPE_CHECKING

from ibm_watsonx_data_integration.services.datastage.models.flow.dag import StageNode
from ibm_watsonx_data_integration.services.datastage.models.stage_models.transformer import transformer


if TYPE_CHECKING:
    from ibm_watsonx_data_integration.services.datastage.models.flow.dag import DAG


import os
# from ibm.datastage._framework.components.function_library import FunctionLibrary


class TransformerStage(StageNode):
    def __init__(self, dag: "DAG", configuration: transformer=None, *, label: str = None):
        super().__init__(dag, configuration=configuration, label=label)
        self.configuration: transformer = configuration or transformer()


    # def set_function_library(self, function_library: FunctionLibrary):
    #    self.configuration.function_library = function_library
    #    self.configuration.function_library_asset_name = function_library.name
    #    self.configuration.function_library_path = function_library.library_path



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import WATSONX_DATA_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class AuthMethod(Enum):
    username_apikey = "username_apikey"
    username_password = "username_password"


class DeploymentType(Enum):
    software_dev = "software_dev"
    saas = "saas"
    software_ent = "software_ent"


class WatsonxDataConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "lakehouse"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    api_key: str = Field(None, alias="api_key")
    authentication_method: WATSONX_DATA_CONNECTION.AuthMethod | None = Field(
        WATSONX_DATA_CONNECTION.AuthMethod.username_password, alias="auth_method"
    )
    cloud_resource_name: str = Field(None, alias="cloud_resource_name")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    crn: str | None = Field(None, alias="crn")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    deployment_type: WATSONX_DATA_CONNECTION.DeploymentType | None = Field(None, alias="deployment_type")
    engine_hostname: str | None = Field(None, alias="engine_host")
    engine_id: str | None = Field(None, alias="engine_id")
    engine_port: int | None = Field(None, alias="engine_port")
    engine_ssl_enabled: bool | None = Field(None, alias="engine_ssl")
    engine_ssl_certificate: str | None = Field(None, alias="engine_ssl_certificate")
    hostname_or_ip_address: str = Field(None, alias="host")
    instance_id: str = Field(None, alias="instance_id")
    instance_name: str = Field(None, alias="instance_name")
    connect_to_ibm_watsonx_data_on_red_hat_open_shift: bool | None = Field(False, alias="is_cpd")
    password: str = Field(None, alias="password")
    port: int = Field(None, alias="port")
    service_to_service_authorization: bool | None = Field(None, alias="s2s")
    ssl_is_enabled: bool | None = Field(True, alias="ssl")
    ssl_certificate: str | None = Field(None, alias="ssl_certificate")
    validate_ssl_certificate: bool | None = Field(None, alias="ssl_certificate_validation")
    username: str = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    ssl_certificate_file: str | None = Field(None, alias="ssl_certificate_file")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        (
            include.add("ssl_certificate")
            if ((((not self.ssl_certificate_file))) and (self.ssl_is_enabled))
            else exclude.add("ssl_certificate")
        )
        include.add("engine_ssl_certificate") if (self.engine_ssl_enabled) else exclude.add("engine_ssl_certificate")
        (
            include.add("password")
            if (
                ((not self.defer_credentials))
                and (
                    ((not self.api_key))
                    and (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value != "username_apikey"
                            )
                            or (self.authentication_method != "username_apikey")
                        )
                    )
                    and (
                        self.deployment_type
                        and (
                            (hasattr(self.deployment_type, "value") and self.deployment_type.value != "saas")
                            or (self.deployment_type != "saas")
                        )
                    )
                )
            )
            else exclude.add("password")
        )
        (
            include.add("authentication_method")
            if (
                ((self.connect_to_ibm_watsonx_data_on_red_hat_open_shift))
                or (
                    self.deployment_type
                    and (
                        (hasattr(self.deployment_type, "value") and self.deployment_type.value == "software_ent")
                        or (self.deployment_type == "software_ent")
                    )
                )
            )
            else exclude.add("authentication_method")
        )
        (
            include.add("instance_name")
            if (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and not self.deployment_type.value)
                    or (not self.deployment_type)
                )
            )
            else exclude.add("instance_name")
        )
        (
            include.add("instance_id")
            if (
                (
                    (
                        self.deployment_type
                        and (
                            (hasattr(self.deployment_type, "value") and not self.deployment_type.value)
                            or (not self.deployment_type)
                        )
                    )
                )
                or (
                    self.deployment_type
                    and (
                        (hasattr(self.deployment_type, "value") and self.deployment_type.value == "software_ent")
                        or (self.deployment_type == "software_ent")
                    )
                )
                or (
                    self.deployment_type
                    and (
                        (hasattr(self.deployment_type, "value") and self.deployment_type.value == "software_dev")
                        or (self.deployment_type == "software_dev")
                    )
                )
            )
            else exclude.add("instance_id")
        )
        (
            include.add("service_to_service_authorization")
            if (
                (
                    (
                        self.deployment_type
                        and (
                            (hasattr(self.deployment_type, "value") and self.deployment_type.value == "saas")
                            or (self.deployment_type == "saas")
                        )
                    )
                )
                and (not self.connect_to_ibm_watsonx_data_on_red_hat_open_shift)
            )
            else exclude.add("service_to_service_authorization")
        )
        (
            include.add("api_key")
            if (
                (
                    ((not self.password))
                    and (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value != "username_password"
                                )
                                or (self.authentication_method != "username_password")
                            )
                        )
                    )
                )
                and (
                    ((self.connect_to_ibm_watsonx_data_on_red_hat_open_shift))
                    or (
                        self.deployment_type
                        and (
                            (hasattr(self.deployment_type, "value") and self.deployment_type.value == "saas")
                            or (self.deployment_type == "saas")
                        )
                    )
                    or (
                        self.deployment_type
                        and (
                            (hasattr(self.deployment_type, "value") and self.deployment_type.value == "software_ent")
                            or (self.deployment_type == "software_ent")
                        )
                    )
                )
            )
            else exclude.add("api_key")
        )
        (
            include.add("cloud_resource_name")
            if (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and self.deployment_type.value == "saas")
                    or (self.deployment_type == "saas")
                )
            )
            else exclude.add("cloud_resource_name")
        )
        (
            include.add("connect_to_ibm_watsonx_data_on_red_hat_open_shift")
            if (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and not self.deployment_type.value)
                    or (not self.deployment_type)
                )
            )
            else exclude.add("connect_to_ibm_watsonx_data_on_red_hat_open_shift")
        )
        include.add("crn") if (not self.connect_to_ibm_watsonx_data_on_red_hat_open_shift) else exclude.add("crn")
        (
            include.add("username")
            if (
                ((not self.defer_credentials))
                and (
                    self.deployment_type
                    and (
                        (hasattr(self.deployment_type, "value") and self.deployment_type.value != "saas")
                        or (self.deployment_type != "saas")
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")
        include.add("ssl_certificate_file") if (self.hidden_dummy_property1) else exclude.add("ssl_certificate_file")

        (
            include.add("instance_id")
            if (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and not self.deployment_type.value)
                    or (not self.deployment_type)
                )
            )
            or (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and self.deployment_type.value == "software_ent")
                    or (self.deployment_type == "software_ent")
                )
            )
            or (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and self.deployment_type.value == "software_dev")
                    or (self.deployment_type == "software_dev")
                )
            )
            else exclude.add("instance_id")
        )
        (
            include.add("api_key")
            if (
                (not self.password)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value != "username_password"
                        )
                        or (self.authentication_method != "username_password")
                    )
                )
            )
            and (
                (
                    self.connect_to_ibm_watsonx_data_on_red_hat_open_shift == "true"
                    or self.connect_to_ibm_watsonx_data_on_red_hat_open_shift
                )
                or (
                    self.deployment_type
                    and (
                        (hasattr(self.deployment_type, "value") and self.deployment_type.value == "saas")
                        or (self.deployment_type == "saas")
                    )
                )
                or (
                    self.deployment_type
                    and (
                        (hasattr(self.deployment_type, "value") and self.deployment_type.value == "software_ent")
                        or (self.deployment_type == "software_ent")
                    )
                )
            )
            else exclude.add("api_key")
        )
        (
            include.add("authentication_method")
            if (
                self.connect_to_ibm_watsonx_data_on_red_hat_open_shift == "true"
                or self.connect_to_ibm_watsonx_data_on_red_hat_open_shift
            )
            or (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and self.deployment_type.value == "software_ent")
                    or (self.deployment_type == "software_ent")
                )
            )
            else exclude.add("authentication_method")
        )
        (
            include.add("cloud_resource_name")
            if (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and self.deployment_type.value == "saas")
                    or (self.deployment_type == "saas")
                )
            )
            else exclude.add("cloud_resource_name")
        )
        (
            include.add("service_to_service_authorization")
            if (
                (
                    self.deployment_type
                    and (
                        (hasattr(self.deployment_type, "value") and self.deployment_type.value == "saas")
                        or (self.deployment_type == "saas")
                    )
                )
                and (not self.connect_to_ibm_watsonx_data_on_red_hat_open_shift)
            )
            else exclude.add("service_to_service_authorization")
        )
        (
            include.add("ssl_certificate_file")
            if (not self.ssl_certificate) and (self.ssl_is_enabled == "true" or self.ssl_is_enabled)
            else exclude.add("ssl_certificate_file")
        )
        (
            include.add("connect_to_ibm_watsonx_data_on_red_hat_open_shift")
            if (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and not self.deployment_type.value)
                    or (not self.deployment_type)
                )
            )
            else exclude.add("connect_to_ibm_watsonx_data_on_red_hat_open_shift")
        )
        (
            include.add("password")
            if (not self.defer_credentials)
            and (
                (not self.api_key)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value != "username_apikey"
                        )
                        or (self.authentication_method != "username_apikey")
                    )
                )
                and (
                    self.deployment_type
                    and (
                        (hasattr(self.deployment_type, "value") and self.deployment_type.value != "saas")
                        or (self.deployment_type != "saas")
                    )
                )
            )
            else exclude.add("password")
        )
        (
            include.add("engine_ssl_certificate")
            if (self.engine_ssl_enabled == "true" or self.engine_ssl_enabled)
            else exclude.add("engine_ssl_certificate")
        )
        (
            include.add("username")
            if (not self.defer_credentials)
            and (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and self.deployment_type.value != "saas")
                    or (self.deployment_type != "saas")
                )
            )
            else exclude.add("username")
        )
        (
            include.add("crn")
            if (
                self.connect_to_ibm_watsonx_data_on_red_hat_open_shift == "false"
                or not self.connect_to_ibm_watsonx_data_on_red_hat_open_shift
            )
            else exclude.add("crn")
        )
        (
            include.add("instance_name")
            if (
                self.deployment_type
                and (
                    (hasattr(self.deployment_type, "value") and not self.deployment_type.value)
                    or (not self.deployment_type)
                )
            )
            else exclude.add("instance_name")
        )
        (
            include.add("ssl_certificate")
            if (not self.ssl_certificate_file) and (self.ssl_is_enabled == "true" or self.ssl_is_enabled)
            else exclude.add("ssl_certificate")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



# IBM Confidential
# PID 5900-BAF
# Copyright StreamSets Inc., an IBM Company 2025

"""This module contains common sources for PythonGenerator."""

from ibm_watsonx_data_integration.codegen.sources.base_source import Source
from ibm_watsonx_data_integration.codegen.sources.flow_object_source import FlowObjectSource
from ibm_watsonx_data_integration.cpd_models.flow_model import Flow


def source_factory(source: str | Flow) -> Source:
    """Factory wrapping source to appropriate object.

    Args:
        source: Input source that contains flow definition.

    Raises:
        TypeError: If input source type is not supported.
    """
    if not isinstance(source, Flow):
        raise TypeError("Currently only Flow class instance is supported as a source.")

    return FlowObjectSource(source)


__all__ = ["source_factory"]



from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestAmazonPostgreSqlConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "amazonpostgresql_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _testtable_1_ = fc.add_stage("Amazon RDS for PostgreSQL", "TESTTABLE_1")
        _testtable_1_.configuration.schema_name = "TM_DS_1"
        _testtable_1_.configuration.table_name = "TESTTABLE"

        _testtable_1_.configuration.connection.database = "conopsdb"
        _testtable_1_.configuration.connection.host = environ["AMAZONPOSTGRESQL_HOST"]
        _testtable_1_.configuration.connection.password = environ["AMAZONPOSTGRESQL_PASSWORD"]
        _testtable_1_.configuration.connection.port = "5432"
        _testtable_1_.configuration.connection.proxy = False
        _testtable_1_.configuration.connection.ssl = True
        _testtable_1_.configuration.connection.username = environ["AMAZONPOSTGRESQL_USERNAME"]

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        testtable_1_schema = _testtable_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()
        testtable_1_schema.add_field("VARCHAR", "COLUMN_1").nullable().length(100)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



from enum import Enum
from pydantic import BaseModel, ConfigDict, Field
from typing import ClassVar


class ConnectionType(Enum):
    application_server = "application_server"
    load_balancing = "load_balancing"
    snc_application_server = "snc_application_server"
    snc_load_balancing = "snc_load_balancing"


class SncQop(Enum):
    auth_only = "1"
    auth_integrity = "2"
    auth_integrity_privacy = "3"
    global_default = "8"
    max_protection = "9"


class SapbapiConn(BaseModel):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "sapbapi"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    connection_type: ConnectionType = Field(ConnectionType.application_server, alias="connection_type")
    jar_uris: str | None = Field(None, alias="jar_uris")
    gateway_url: str | None = Field(None, alias="gateway_url")
    sap_application_server: str = Field(None, alias="sap_application_server")
    sap_application_system_number: str = Field(None, alias="sap_application_system_number")
    message_server: str = Field(None, alias="message_server")
    use_system_number: bool | None = Field(True, alias="use_system_number")
    system_number: str = Field(None, alias="system_number")
    system_id: str = Field(None, alias="system_id")
    group: str = Field(None, alias="group")
    sap_router: str | None = Field(None, alias="sap_router")
    snc_name: str = Field(None, alias="snc_name")
    snc_partner_name: str = Field(None, alias="snc_partner_name")
    snc_qop: SncQop | None = Field(SncQop.auth_integrity_privacy, alias="snc_qop")
    custom_namespace: str | None = Field("", alias="custom_namespace")
    isx509enabled: bool | None = Field(False, alias="isx509enabled")
    x509_cert: str = Field(None, alias="x509_cert")
    username: str = Field(None, alias="username")
    password: str = Field(None, alias="password")
    client_number: str = Field(None, alias="client_number")
    language: str = Field(None, alias="language")
    cluster_access_token: str = Field(None, alias="cluster_access_token")
    cluster_user_name: str = Field(None, alias="cluster_user_name")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        (include.add("client_number") if (not self.defer_credentials) else exclude.add("client_number"))
        (
            include.add("system_number")
            if (self.use_system_number)
            and (
                (self.connection_type.value == "load_balancing") or (self.connection_type.value == "snc_load_balancing")
            )
            else exclude.add("system_number")
        )
        (
            include.add("use_system_number")
            if (
                (self.connection_type.value == "load_balancing") or (self.connection_type.value == "snc_load_balancing")
            )
            else exclude.add("use_system_number")
        )
        (
            include.add("system_id")
            if (not self.use_system_number)
            and (
                (self.connection_type.value == "load_balancing") or (self.connection_type.value == "snc_load_balancing")
            )
            else exclude.add("system_id")
        )
        (
            include.add("sap_application_system_number")
            if (
                (self.connection_type.value == "application_server")
                or (self.connection_type.value == "snc_application_server")
            )
            else exclude.add("sap_application_system_number")
        )
        (include.add("language") if (not self.defer_credentials) else exclude.add("language"))
        (
            include.add("isx509enabled")
            if (
                (self.connection_type.value == "snc_application_server")
                or (self.connection_type.value == "snc_load_balancing")
            )
            else exclude.add("isx509enabled")
        )
        (
            include.add("snc_partner_name")
            if (
                (self.connection_type.value == "snc_application_server")
                or (self.connection_type.value == "snc_load_balancing")
            )
            else exclude.add("snc_partner_name")
        )
        (
            include.add("snc_name")
            if (
                (self.connection_type.value == "snc_application_server")
                or (self.connection_type.value == "snc_load_balancing")
            )
            else exclude.add("snc_name")
        )
        (
            include.add("password")
            if (not self.defer_credentials)
            and (
                (
                    (self.connection_type.value == "application_server")
                    or (self.connection_type.value == "load_balancing")
                )
                or (not self.isx509enabled)
            )
            else exclude.add("password")
        )
        (
            include.add("sap_application_server")
            if (
                (self.connection_type.value == "application_server")
                or (self.connection_type.value == "snc_application_server")
            )
            else exclude.add("sap_application_server")
        )
        (
            include.add("message_server")
            if (
                (self.connection_type.value == "load_balancing") or (self.connection_type.value == "snc_load_balancing")
            )
            else exclude.add("message_server")
        )
        (
            include.add("x509_cert")
            if (self.isx509enabled)
            and (
                (self.connection_type.value == "snc_application_server")
                or (self.connection_type.value == "snc_load_balancing")
            )
            else exclude.add("x509_cert")
        )
        (
            include.add("snc_qop")
            if (
                (self.connection_type.value == "snc_application_server")
                or (self.connection_type.value == "snc_load_balancing")
            )
            else exclude.add("snc_qop")
        )
        (
            include.add("group")
            if (
                (self.connection_type.value == "load_balancing") or (self.connection_type.value == "snc_load_balancing")
            )
            else exclude.add("group")
        )
        (
            include.add("username")
            if (not self.defer_credentials)
            and (
                (
                    (self.connection_type.value == "application_server")
                    or (self.connection_type.value == "load_balancing")
                )
                or (not self.isx509enabled)
            )
            else exclude.add("username")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )

        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls(**properties)



import collections
import typing
from ibm_watsonx_data_integration.services.datastage.models.flow.dag import (
    DAG,
    Link,
    Node,
    StageNode,
)
from ibm_watsonx_data_integration.services.datastage.models.layout.common import GroupBoundingBox, NodeBoundingBox

NodeLayer = list[Node]


class LayeredLayout:
    # Default node dimensions
    NODE_WIDTH = 48
    NODE_HEIGHT = 48
    COMMENT_WIDTH = 175
    COMMENT_HEIGHT = 60

    # Constraints for spacing
    NODE_SPACING = 200
    LAYER_SPACING = 200

    # Estimated center of the canvas
    CANVAS_WIDTH = 1600
    CANVAS_HEIGHT = 800

    # Number of iterations of node-swapping to minimize edge crossings. One iteration consists of two sweeps
    BARYCENTRIC_SWEEP_ITERATIONS = 2

    # Number of iterations to refine node positions
    REFINEMENT_ITERATIONS = 300

    def __init__(self, dag: DAG):
        self.dag = dag
        self.layers: list[NodeLayer] = []
        #self.comments: list[CommentNode] = []
        self.node_boxes: dict[Node, NodeBoundingBox] = {}
        self.master_container = GroupBoundingBox()

    def compute(self):
        """Compute the layered layout of the DAG, populating the x, y, and height (if non-default) properties of each node."""
        # self.__collect_comment_nodes()
        # self.__insert_ghost_nodes()
        self.__collect_layers()
        self.__order_layer_nodes()

        self.__position_nodes()

        for _ in range(self.REFINEMENT_ITERATIONS):
            self.__apply_forces()

        # self.__remove_ghost_nodes()
        self.master_container.center_within(GroupBoundingBox(width=self.CANVAS_WIDTH, height=self.CANVAS_HEIGHT))
        self.master_container.update_absolute_positions(0, 0)

    # def __collect_comment_nodes(self):
    #     """Collect comment nodes from the DAG and set their dimensions."""
    #     for node in self.dag.nodes():
    #         if not isinstance(node, CommentNode):
    #             continue
    #         self.comments.append(node)

    # def __insert_ghost_nodes(self):
    #     """Insert ghost nodes for edges that span non-adjacent layers. This helps reduce edge crossings caused by
    #     long edges that span multiple layers.

    #     For each edge from a parent to a child where child's layer > parent's layer + 1, create ghost nodes for each
    #     intermediate layer. Each ghost node is marked with metadata['ghost'] = True.
    #     """
    #     for parent in list(self.dag.nodes()):
    #         # Not necessary for comment nodes, as we will position these later
    #         if isinstance(parent, CommentNode):
    #             continue

    #         for child in list(parent.metadata["children"]):
    #             parent_layer_num = parent.metadata["layer"]
    #             child_layer_num = child.metadata["layer"]

    #             # Check if the edge spans more than one layer
    #             if child_layer_num > parent_layer_num + 1:
    #                 last: Node = parent
    #                 # Insert ghost nodes for each intermediate layer
    #                 for layer_num in range(parent_layer_num + 1, child_layer_num):
    #                     ghost_node = GhostNode()
    #                     ghost_node.metadata["layer"] = layer_num
    #                     ghost_node.metadata["children"] = []
    #                     ghost_node.metadata["parents"] = [last]
    #                     # Connect the previous node to the ghost node
    #                     self.dag.adj.setdefault(last, {})[ghost_node] = []
    #                     last.metadata["children"].append(ghost_node)
    #                     last = ghost_node

    #                 # Connect the last ghost node to the child node
    #                 last.metadata["children"].append(child)
    #                 child.metadata["parents"].append(last)
    #                 self.dag.adj.setdefault(last, {})[child] = []

    # def __remove_ghost_nodes(self):
    #     """Remove previously inserted ghost nodes from the DAG."""
    #     clean_adj: dict[Node, dict[Node, list[Link]]] = {}

    #     for from_node in self.dag.nodes():
    #         if from_node.metadata.get("ghost", False):
    #             continue

    #         # Add non-ghost parent nodes to the new adjacency list
    #         # If the parent has any non-ghost children, their links will also be added
    #         clean_adj.setdefault(from_node, {})
    #         for to_node, links in self.dag.adj[from_node].items():
    #             if not to_node.metadata.get("ghost", False):
    #                 # Only keep links where both the parent and child are not ghost nodes
    #                 clean_adj[from_node][to_node] = links

    #     self.dag.adj = clean_adj

    def __apply_forces(self):
        """Applies two sets of forces:
        1. Weak attractive forces from nodes to their parents
        2. Strong repulsive forces between neighboring nodes in the same layer
        """
        for layer_box in self.master_container.children:
            layer_box = typing.cast(GroupBoundingBox, layer_box)
            for node_box in layer_box.children:
                node_box = typing.cast(NodeBoundingBox, node_box)
                node = node_box.node

                for parent in node.metadata["parents"]:
                    parent_box = self.node_boxes[parent]
                    # TODO: Diminishing force based on layer proximity
                    node_box.add_force(0, min(80, (parent_box.center_y() - node_box.center_y()) * 0.2))

        # Apply repulsive forces between neighboring nodes in the same layer
        for layer_box in self.master_container.children:
            layer_box = typing.cast(GroupBoundingBox, layer_box)
            for i, node_box in enumerate(layer_box.children):
                node_box = typing.cast(NodeBoundingBox, node_box)
                for j, other_node_box in enumerate(layer_box.children):
                    if i == j:
                        continue
                    other_node_box = typing.cast(NodeBoundingBox, other_node_box)
                    # Apply force that exponentially grows as the nodes get closer to MIN_NODE_SPACING
                    sign = -1 if node_box.center_y() < other_node_box.center_y() else 1
                    dist = node_box.y_boundary_distance(other_node_box)
                    # Only apply force if nodes are close enough
                    if dist < 300:
                        force = (600 / max(dist, 1)) ** 2
                        node_box.add_force(0, sign * force)

        for layer_box in self.master_container.children:
            layer_box = typing.cast(GroupBoundingBox, layer_box)
            for node_box in layer_box.children:
                node_box = typing.cast(NodeBoundingBox, node_box)
                node_box.apply_forces()

    @staticmethod
    def __count_node_relatives(node: Node):
        """Count the number of relatives (parents and children) of a node, excluding ghost nodes."""

        def is_relative(n: Node):
            return not n.metadata.get("ghost", False) #and not isinstance(n, CommentNode)

        parent_count = len([n for n in node.metadata["parents"] if is_relative(n)])
        child_count = len([n for n in node.metadata["children"] if is_relative(n)])

        return max(parent_count, child_count)

    def __position_nodes(self):
        """Position nodes based on their layer number and order within the layer as an initial starting point.
        Also adjust the height of nodes with many relatives to improve visual distinction.
        """
        for layer_num, layer in enumerate(self.layers):
            has_comment = False
            layer_container = GroupBoundingBox()

            for node_num, node in enumerate(layer):
                # Resize non-comment nodes if conditions are met
                if isinstance(node, StageNode):
                    width = self.NODE_WIDTH
                    height = self.NODE_HEIGHT
                    if (relatives := self.__count_node_relatives(node)) >= 3:
                        # Increase height of nodes with many relatives (either many parents or many children)
                        height = (self.NODE_HEIGHT + 10) * relatives
                else:
                    width = self.COMMENT_WIDTH
                    height = self.COMMENT_HEIGHT
                    has_comment = True

                box = NodeBoundingBox(node=node, width=width, height=height)
                self.node_boxes[node] = box
                layer_container.children.append(box)

            layer_container.width = self.COMMENT_WIDTH if has_comment else self.NODE_WIDTH
            layer_container.arrange_children_vertically(self.NODE_SPACING)
            self.master_container.children.append(layer_container)
            self.master_container.arrange_children_horizontally(self.LAYER_SPACING)

    def __order_layer_nodes(self):
        """Order nodes within each layer based on the barycentric heuristic.

        The barycentric heuristic reduces edge crossings by ordering nodes in one layer based on the average order of
        their neighbors. We perform alternating downward and upward sweeps to iteratively improve the ordering of nodes
        within each layer.
        """
        # Initially order nodes within each layer alphabetically for consistency
        for layer in self.layers:
            layer.sort(key=lambda n: type(n).__name__)

        # Then order nodes based on barycentric heuristic
        for _ in range(self.BARYCENTRIC_SWEEP_ITERATIONS):
            # Downward sweep - order layers from top to bottom
            for i in range(1, len(self.layers)):
                barycenters = {}
                for node in self.layers[i]:
                    barycenters[node] = self.__compute_weighted_barycenter(node, i)
                self.layers[i].sort(key=lambda n: barycenters[n])

            # Upward sweep - order layers from bottom to top
            for i in range(len(self.layers) - 2, -1, -1):
                barycenters = {}
                for node in self.layers[i]:
                    barycenters[node] = self.__compute_weighted_barycenter(node, i)
                self.layers[i].sort(key=lambda n: barycenters[n])

    def __compute_normalized_index(self, node: Node, current_layer_num: int):
        """Computes the continuous normalized index of a node within its layer.
        0.5 means the node is in the middle of the layer.
        0 means the node is the first in the layer.
        1 means the node is last.
        """
        layer = self.layers[current_layer_num]
        if len(layer) == 1:
            return 0.5
        return layer.index(node) / (len(layer) - 1)

    def __compute_weighted_barycenter(self, node: Node, current_layer_num: int):
        """Computes the barycenter of a node within its layer. The computation discounts the influence of neighbor nodes
        that are in layers further away from the current layer.
        """
        total_weight = 0
        weighted_sum = 0

        for parent in node.metadata["parents"]:
            parent_layer_num = parent.metadata["layer"]
            if parent_layer_num < current_layer_num:
                weight = 1 / (current_layer_num - parent_layer_num)
                weighted_sum += weight * self.__compute_normalized_index(parent, parent_layer_num)
                total_weight += weight

        for child in node.metadata["children"]:
            child_layer_num = child.metadata["layer"]
            if child_layer_num > current_layer_num:
                weight = 1 / (child_layer_num - current_layer_num)
                weighted_sum += weight * self.__compute_normalized_index(child, child_layer_num)
                total_weight += weight

        if total_weight > 0:
            return weighted_sum / total_weight
        else:
            return 0.5

    def __collect_layers(self):
        """Collect nodes into list of layers based on the topological order of the DAG."""
        assert self.dag.adj, "DAG is empty"

        # Collect nodes into a dictionary mapping the layer # to the Layer
        layer_dict: dict[int, NodeLayer] = collections.defaultdict(list)
        max_layer = 0
        for node in self.dag.nodes():
            # Don't include comment nodes in layers yet
            # if isinstance(node, CommentNode):
            #     continue
            layer_num = node.metadata["layer"]
            max_layer = max(max_layer, layer_num)
            layer_dict[layer_num].append(node)

        # Assign comment nodes to layers
        # for comment in self.comments:
        #     children = comment.metadata["children"]
        #     if not children:
        #         layer_dict[0].append(comment)
        #         continue

        #     # Put comment node in layer upstream to children
        #     layer_num = min([child.metadata["layer"] for child in children]) - 1
        #     comment.metadata["layer"] = layer_num
        #     layer_dict[layer_num].append(comment)

        # Convert the dictionary mapping to an ordered list of Layers
        for layer_num in range(max_layer + 1):
            self.layers.append(layer_dict[layer_num])



from ibm_watsonx_data_integration.services.datastage.models.enums import AZURE_POSTGRESQL_CONNECTION
from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestAzurePosgresqlConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "azurepostgresql_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _test1_1_ = fc.add_stage("Azure PostgreSQL", "Test1_1")
        _test1_1_.configuration.schema_name = "public"
        _test1_1_.configuration.table_name = "Test1"

        _test1_1_.configuration.connection.authentication_method = (
            AZURE_POSTGRESQL_CONNECTION.AuthMethod.user_credentials
        )
        _test1_1_.configuration.connection.database = "postgres"
        _test1_1_.configuration.connection.hostname_or_ip_address = environ["AZUREPOSTGRESQL_HOST"]
        _test1_1_.configuration.connection.password = environ["AZUREPOSTGRESQL_PASSWORD"]
        _test1_1_.configuration.connection.port = "5432"
        _test1_1_.configuration.connection.proxy = False
        _test1_1_.configuration.connection.port_is_ssl_enabled = True
        _test1_1_.configuration.connection.username = environ["AZUREPOSTGRESQL_USERNAME"]

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        test1_1_schema = _test1_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        test1_1_schema.add_field("CHAR", "col1").nullable().length(6)
        test1_1_schema.add_field("NUMERIC", "col2").nullable().length(6)
        test1_1_schema.add_field("VARCHAR", "col3").nullable().length(6)
        test1_1_schema.add_field("DOUBLE", "col4").nullable()
        test1_1_schema.add_field("BIGINT", "col6").nullable()
        test1_1_schema.add_field("LONGVARBINARY", "col7").nullable().length(2048)
        test1_1_schema.add_field("DOUBLE", "col8").nullable()
        test1_1_schema.add_field("BIT", "col9").nullable()
        test1_1_schema.add_field("DATE", "col10").nullable()
        test1_1_schema.add_field("INTEGER", "col11").nullable()
        test1_1_schema.add_field("VARCHAR", "col12").nullable().length(6)
        test1_1_schema.add_field("VARCHAR", "col13").nullable().length(6)
        test1_1_schema.add_field("LONGVARBINARY", "col14").nullable().length(2048)
        test1_1_schema.add_field("CHAR", "col15").nullable().length(6)
        test1_1_schema.add_field("NUMERIC", "col16").nullable().length(6)
        test1_1_schema.add_field("VARCHAR", "col17").nullable().length(6)
        test1_1_schema.add_field("REAL", "col18").nullable().length(7)
        test1_1_schema.add_field("SMALLINT", "col19").nullable()
        test1_1_schema.add_field("TIMESTAMP", "col20").nullable()
        test1_1_schema.add_field("TIMESTAMP", "col21").nullable()
        test1_1_schema.add_field("SMALLINT", "col22").nullable()
        test1_1_schema.add_field("LONGVARBINARY", "col23").nullable().length(2048)
        test1_1_schema.add_field("VARCHAR", "col24").nullable().length(6)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import CLOUD_OBJECT_STORAGE_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class AuthMethod(Enum):
    accesskey_secretkey = "accesskey_secretkey"
    instanceid_apikey = "instanceid_apikey"
    instanceid_apikey_accesskey_secretkey = "instanceid_apikey_accesskey_secretkey"
    credentials = "credentials"


class CloudObjectStorageConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "193a97c1-4475-4a19-b90c-295c4fdc6517"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    ds_host: str | None = Field(None, alias="_host")
    ds_port: str | None = Field(None, alias="_port")
    access_key: str | None = Field(None, alias="access_key")
    api_key: str = Field(None, alias="api_key")
    authentication_method: CLOUD_OBJECT_STORAGE_CONNECTION.AuthMethod | None = Field(None, alias="auth_method")
    discover_data_assets: bool | None = Field(None, alias="auto_discovery")
    bucket: str | None = Field(None, alias="bucket")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    service_credentials: str | None = Field(None, alias="credentials")
    credentials_file_path: str | None = Field(None, alias="credentials_file")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    iam_cos_url: str | None = Field(None, alias="iam_cos_url")
    iam_url: str | None = Field(None, alias="iam_url")
    region_deprecated: str | None = Field(None, alias="region")
    resource_instance_id: str | None = Field(None, alias="resource_instance_id")
    secret_key: str = Field(None, alias="secret_key")
    ssl_certificate: str | None = Field(None, alias="ssl_certificate")
    validate_ssl_certificate: bool | None = Field(None, alias="ssl_certificate_validation")
    trust_all_ssl_certificates: bool | None = Field(False, alias="trust_all_ssl_cert")
    url: str = Field(None, alias="url")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    ssl_certificate_file: str | None = Field(None, alias="ssl_certificate_file")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        include.add("ssl_certificate") if (not self.ssl_certificate_file) else exclude.add("ssl_certificate")
        (
            include.add("secret_key")
            if (
                (
                    ((not self.defer_credentials))
                    and ((self.access_key))
                    and ((not self.service_credentials))
                    and ((not self.credentials_file_path))
                    and (
                        (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "accesskey_secretkey"
                                    )
                                    or (self.authentication_method == "accesskey_secretkey")
                                )
                            )
                        )
                        or (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "instanceid_apikey_accesskey_secretkey"
                                    )
                                    or (self.authentication_method == "instanceid_apikey_accesskey_secretkey")
                                )
                            )
                        )
                    )
                )
                or (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "accesskey_secretkey"
                                )
                                or (self.authentication_method == "accesskey_secretkey")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "instanceid_apikey_accesskey_secretkey"
                            )
                            or (self.authentication_method == "instanceid_apikey_accesskey_secretkey")
                        )
                    )
                )
            )
            else exclude.add("secret_key")
        )
        (
            include.add("credentials_file_path")
            if (
                ((not self.defer_credentials))
                and (not self.access_key)
                and (not self.api_key)
                and (not self.service_credentials)
                and (not self.resource_instance_id)
                and (not self.secret_key)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "credentials_file"
                        )
                        or (self.authentication_method == "credentials_file")
                    )
                )
            )
            else exclude.add("credentials_file_path")
        )
        (
            include.add("service_credentials")
            if (
                ((not self.defer_credentials))
                and (not self.access_key)
                and (not self.api_key)
                and (not self.credentials_file_path)
                and (not self.resource_instance_id)
                and (not self.secret_key)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "credentials"
                        )
                        or (self.authentication_method == "credentials")
                    )
                )
            )
            else exclude.add("service_credentials")
        )
        (
            include.add("api_key")
            if (
                (
                    ((not self.defer_credentials))
                    and ((not self.service_credentials))
                    and ((not self.credentials_file_path))
                    and ((self.resource_instance_id))
                    and (
                        (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "instanceid_apikey"
                                    )
                                    or (self.authentication_method == "instanceid_apikey")
                                )
                            )
                        )
                        or (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "instanceid_apikey_accesskey_secretkey"
                                    )
                                    or (self.authentication_method == "instanceid_apikey_accesskey_secretkey")
                                )
                            )
                        )
                    )
                )
                or (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "instanceid_apikey"
                                )
                                or (self.authentication_method == "instanceid_apikey")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "instanceid_apikey_accesskey_secretkey"
                            )
                            or (self.authentication_method == "instanceid_apikey_accesskey_secretkey")
                        )
                    )
                )
            )
            else exclude.add("api_key")
        )
        (
            include.add("resource_instance_id")
            if (
                ((not self.defer_credentials))
                and (not self.service_credentials)
                and (not self.credentials_file_path)
                and (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "instanceid_apikey"
                                )
                                or (self.authentication_method == "instanceid_apikey")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "instanceid_apikey_accesskey_secretkey"
                            )
                            or (self.authentication_method == "instanceid_apikey_accesskey_secretkey")
                        )
                    )
                )
            )
            else exclude.add("resource_instance_id")
        )
        (
            include.add("access_key")
            if (
                ((not self.defer_credentials))
                and (not self.service_credentials)
                and (not self.credentials_file_path)
                and (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "accesskey_secretkey"
                                )
                                or (self.authentication_method == "accesskey_secretkey")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "instanceid_apikey_accesskey_secretkey"
                            )
                            or (self.authentication_method == "instanceid_apikey_accesskey_secretkey")
                        )
                    )
                )
            )
            else exclude.add("access_key")
        )
        include.add("region_deprecated") if (self.resource_instance_id) else exclude.add("region_deprecated")
        (
            include.add("api_key")
            if (
                ((not self.defer_credentials))
                and (not self.service_credentials)
                and (not self.credentials_file_path)
                and (self.resource_instance_id)
                and (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "instanceid_apikey"
                                )
                                or (self.authentication_method == "instanceid_apikey")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "instanceid_apikey_accesskey_secretkey"
                            )
                            or (self.authentication_method == "instanceid_apikey_accesskey_secretkey")
                        )
                    )
                )
            )
            else exclude.add("api_key")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("credentials_file_path") if (self.hidden_dummy_property1) else exclude.add("credentials_file_path")
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")
        include.add("ssl_certificate_file") if (self.hidden_dummy_property1) else exclude.add("ssl_certificate_file")

        include.add("region_deprecated") if (self.resource_instance_id) else exclude.add("region_deprecated")
        (
            include.add("api_key")
            if (
                (not self.defer_credentials)
                and (not self.service_credentials)
                and (not self.credentials_file_path)
                and (self.resource_instance_id)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "instanceid_apikey" in str(self.authentication_method.value)
                        )
                        or ("instanceid_apikey" in str(self.authentication_method))
                    )
                    and self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "instanceid_apikey_accesskey_secretkey" in str(self.authentication_method.value)
                        )
                        or ("instanceid_apikey_accesskey_secretkey" in str(self.authentication_method))
                    )
                )
            )
            or (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "instanceid_apikey" in str(self.authentication_method.value)
                    )
                    or ("instanceid_apikey" in str(self.authentication_method))
                )
                or self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "instanceid_apikey_accesskey_secretkey" in str(self.authentication_method.value)
                    )
                    or ("instanceid_apikey_accesskey_secretkey" in str(self.authentication_method))
                )
            )
            else exclude.add("api_key")
        )
        (
            include.add("secret_key")
            if (
                (not self.defer_credentials)
                and (self.access_key)
                and (not self.service_credentials)
                and (not self.credentials_file_path)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "accesskey_secretkey" in str(self.authentication_method.value)
                        )
                        or ("accesskey_secretkey" in str(self.authentication_method))
                    )
                    and self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "instanceid_apikey_accesskey_secretkey" in str(self.authentication_method.value)
                        )
                        or ("instanceid_apikey_accesskey_secretkey" in str(self.authentication_method))
                    )
                )
            )
            or (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "accesskey_secretkey" in str(self.authentication_method.value)
                    )
                    or ("accesskey_secretkey" in str(self.authentication_method))
                )
                or self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "instanceid_apikey_accesskey_secretkey" in str(self.authentication_method.value)
                    )
                    or ("instanceid_apikey_accesskey_secretkey" in str(self.authentication_method))
                )
            )
            else exclude.add("secret_key")
        )
        (
            include.add("credentials_file_path")
            if (not self.defer_credentials)
            and (not self.access_key)
            and (not self.api_key)
            and (not self.service_credentials)
            and (not self.resource_instance_id)
            and (not self.secret_key)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "credentials_file" in str(self.authentication_method.value)
                    )
                    or ("credentials_file" in str(self.authentication_method))
                )
            )
            else exclude.add("credentials_file_path")
        )
        include.add("ssl_certificate_file") if (not self.ssl_certificate) else exclude.add("ssl_certificate_file")
        include.add("ssl_certificate") if (not self.ssl_certificate_file) else exclude.add("ssl_certificate")
        (
            include.add("resource_instance_id")
            if (not self.defer_credentials)
            and (not self.service_credentials)
            and (not self.credentials_file_path)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "instanceid_apikey" in str(self.authentication_method.value)
                    )
                    or ("instanceid_apikey" in str(self.authentication_method))
                )
                and self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "instanceid_apikey_accesskey_secretkey" in str(self.authentication_method.value)
                    )
                    or ("instanceid_apikey_accesskey_secretkey" in str(self.authentication_method))
                )
            )
            else exclude.add("resource_instance_id")
        )
        (
            include.add("service_credentials")
            if (not self.defer_credentials)
            and (not self.access_key)
            and (not self.api_key)
            and (not self.credentials_file_path)
            and (not self.resource_instance_id)
            and (not self.secret_key)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "credentials" in str(self.authentication_method.value)
                    )
                    or ("credentials" in str(self.authentication_method))
                )
            )
            else exclude.add("service_credentials")
        )
        (
            include.add("access_key")
            if (not self.defer_credentials)
            and (not self.service_credentials)
            and (not self.credentials_file_path)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "accesskey_secretkey" in str(self.authentication_method.value)
                    )
                    or ("accesskey_secretkey" in str(self.authentication_method))
                )
                and self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "instanceid_apikey_accesskey_secretkey" in str(self.authentication_method.value)
                    )
                    or ("instanceid_apikey_accesskey_secretkey" in str(self.authentication_method))
                )
            )
            else exclude.add("access_key")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from pydantic import ConfigDict, Field
from typing import ClassVar


class StorageVolumeConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "9f30e3c3-b854-4144-b5c3-98b7a835dc79"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    ds_host: str | None = Field(None, alias="_host")
    ds_port: str | None = Field(None, alias="_port")
    access_token: str = Field(None, alias="access_token")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    gateway_url: str = Field(None, alias="gateway_url")
    inherit_access_token: bool | None = Field(False, alias="inherit_access_token")
    instance_id: str | None = Field(None, alias="instance_id")
    password: str | None = Field(None, alias="password")
    pvc: str | None = Field(None, alias="pvc")
    read_timeout: int | None = Field(300, alias="read_timeout")
    ssl_certificate: str | None = Field(None, alias="ssl_certificate")
    trust_all_ssl_cert: bool | None = Field(False, alias="trust_all_ssl_cert")
    username: str | None = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    volume: str = Field(None, alias="volume")
    ssl_certificate_file: str | None = Field(None, alias="ssl_certificate_file")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        include.add("ssl_certificate") if (not self.ssl_certificate_file) else exclude.add("ssl_certificate")
        (
            include.add("access_token")
            if (((not self.defer_credentials)) and (not self.username))
            else exclude.add("access_token")
        )
        (
            include.add("password")
            if (((not self.defer_credentials)) and (not self.access_token) and (not self.inherit_access_token))
            else exclude.add("password")
        )
        (
            include.add("username")
            if (((not self.defer_credentials)) and (not self.access_token) and (not self.inherit_access_token))
            else exclude.add("username")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("ssl_certificate_file") if (self.hidden_dummy_property1) else exclude.add("ssl_certificate_file")

        include.add("ssl_certificate_file") if (not self.ssl_certificate) else exclude.add("ssl_certificate_file")
        (
            include.add("access_token")
            if (not self.defer_credentials) and (not self.username)
            else exclude.add("access_token")
        )
        (
            include.add("password")
            if (not self.defer_credentials) and (not self.access_token) and (not self.inherit_access_token)
            else exclude.add("password")
        )
        (
            include.add("username")
            if (not self.defer_credentials) and (not self.access_token) and (not self.inherit_access_token)
            else exclude.add("username")
        )
        include.add("ssl_certificate") if (not self.ssl_certificate_file) else exclude.add("ssl_certificate")
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



from ibm_watsonx_data_integration.services.datastage.models.enums import LOOKUP
from pydantic import BaseModel, ConfigDict, Field


class LookupDerivation(BaseModel):
    model_config = ConfigDict(validate_by_name=True, validate_by_alias=True)

    lookup_failure: LOOKUP.LookupFail = Field(LOOKUP.LookupFail.fail, alias="lookupFail")
    reference_link: str = Field(None, alias="reference_link")
    condition: str | None = Field(None, alias="Condition")
    condition_not_met: LOOKUP.ConditionNotMet | None = Field(LOOKUP.ConditionNotMet.fail, alias="conditionNotMet")
    derivations: list[dict] | None = Field([], alias="derivations")


class lookup:
    LookupDerivation = LookupDerivation
